
from systemrdl import RDLListener

class PreExportListener(RDLListener):
    def __init__(self, exporter):
        self.exporter = exporter

        # Max width in bits
        self.max_width_stack = []

    def enter_Addrmap(self, node):
        self.enter_group(node)

    def exit_Addrmap(self, node):
        self.exit_group(node)

    def enter_Regfile(self, node):
        self.enter_group(node)

    def exit_Regfile(self, node):
        self.exit_group(node)

    def enter_Reg(self, node):
        # Update max width in stack
        self.max_width_stack[-1] = max(node.get_property("accesswidth"), self.max_width_stack[-1])

    def enter_Mem(self, node):
        # Update max width in stack
        self.max_width_stack[-1] = max(node.get_property("memwidth"), self.max_width_stack[-1])


    def enter_group(self, node): # pylint: disable=unused-argument
        self.max_width_stack.append(0)

    def exit_group(self, node):
        max_width = self.max_width_stack.pop()

        # Register this node in the bus_width_db
        self.exporter.bus_width_db[node.get_path()] = max_width

        # Propagate max width to parent
        if self.max_width_stack:
            self.max_width_stack[-1] = max(max_width, self.max_width_stack[-1])



import os
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


with open(os.path.join("peakrdl/uvm", "__about__.py")) as f:
    v_dict = {}
    exec(f.read(), v_dict)
    version = v_dict['__version__']

setuptools.setup(
    name="peakrdl-uvm",
    version=version,
    author="Alex Mykyta, Jude Zhang",
    description="Generate UVM register model from compiled SystemRDL input",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SystemRDL/PeakRDL-uvm",
    packages=['peakrdl.uvm'],
    include_package_data=True,
    install_requires=[
        "systemrdl-compiler>=1.12.0",
        "jinja2",
    ],
    classifiers=(
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
    ),
    project_urls={
        "Source": "https://github.com/SystemRDL/PeakRDL-uvm",
        "Tracker": "https://github.com/SystemRDL/PeakRDL-uvm/issues"
    },
)
